#include <avr/io.h>
#include <Arduino.h>
#include "MTPowerMotor.h"

MT_DCPowerMotor::MT_DCPowerMotor(uint8_t pwmmode_param) {
	pwmmode = pwmmode_param;
	state   = RELEASE;
	speed   = 0;
	//  
	pinMode(PINPWMH,    OUTPUT);  //    
	pinMode(PINPWML,    OUTPUT);  //    
	pinMode(PINPHASE,   OUTPUT);  //      -      -  
	pinMode(PINSR,      OUTPUT);  //      -      Mosfet-x   Mosfet-
	pinMode(PINFF1,     INPUT);   //    A3941
	pinMode(PINFF2,     INPUT);   //    A3941
	digitalWrite(PINSR, HIGH);
	apply();
}

void MT_DCPowerMotor::run(uint8_t state_param) {
	state = state_param;
	apply();
}

void MT_DCPowerMotor::setSpeed(uint8_t speed_param) {
	speed = speed_param;
	apply();
}

void MT_DCPowerMotor::apply()
{
	switch (state) {
	case FORWARD:
	case BACKWARD:
		switch (pwmmode) {
		case PWMH:
			analogWrite (PINPWMH,  speed);
			digitalWrite(PINPWML,  HIGH);
			digitalWrite(PINPHASE, state==FORWARD ? HIGH : LOW);
			break;
		case PWML:
			digitalWrite(PINPWMH,  HIGH);
			analogWrite (PINPWML,  speed);
			digitalWrite(PINPHASE, state==FORWARD ? HIGH : LOW);
			break;
		case PWMPHASE:
			digitalWrite(PINPWMH,  HIGH);
			digitalWrite(PINPWML,  HIGH);
			analogWrite (PINPHASE, state==FORWARD ? 128 + speed/2 : 127 - speed/2);
			break;
		}
		break;
	case BRAKE:
		digitalWrite(PINPWMH,  LOW);
		digitalWrite(PINPWML,  HIGH);
//		digitalWrite(PINPHASE, LOW);
		break;
	case RELEASE:
		digitalWrite(PINPWMH,  LOW);
		digitalWrite(PINPWML,  LOW);
		break;
	}
}

// http://playground.arduino.cc/Code/PwmFrequency
// setPwmFrequency(PinPWMH,  1); //       PinPWMH   (1024 - 30,52 Hz; 256 - 122,07 Hz; 64 - 488,28 Hz; 8 - 3906,25 Hz; 1 - 31250,00 Hz)
// setPwmFrequency(PinPWML,  1); //       PinPWML   (1024 - 61,04 Hz; 256 - 244,14 Hz; 64 - 976,56 Hz; 8 - 7812,50 Hz; 1 - 62500,00 Hz)
// setPwmFrequency(PinPhase, 8); //      PinPhase   (1024 - 61,04 Hz; 256 - 244,14 Hz; 64 - 976,56 Hz; 8 - 7812,50 Hz; 1 - 62500,00 Hz)
void MT_DCPowerMotor::setPwmFrequency(int pin, int divisor) {
  uint8_t mode;
  if (pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch (divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if (pin == 5 || pin == 6) {
      TCCR0B = TCCR0B & 0b11111000 | mode;
    } else {
      TCCR1B = TCCR1B & 0b11111000 | mode;
    }
  } else if (pin == 3 || pin == 11) {
    switch (divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x07; break;
      default: return;
    }
    TCCR2B = TCCR2B & 0b11111000 | mode;
  }
}

float MT_DCPowerMotor::getI(int analogInPin) {
  return (analogRead(analogInPin)-511)*30.0f/512;
}
